package  
{
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.events.MouseEvent;
	import flash.geom.Matrix;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.ui.Keyboard;
	/**
	 * ...
	 * @author Shiu
	 */
	[SWF(width = 500, height = 400)]
	public class CollisionTime extends Sprite
	{
		private var w1:Circle, w2:Circle, collision:Circle;
		private var ship:Triangle, velo:Vector2D;
		private var t:TextField;
		private var control:int = 0, increase:int = 0;	
		//0 to maintain course, 1 to steer left, 2 to steer right
		//0 to decrease to initial speed, 1 to increase
		
		
		
		public function CollisionTime() 
		{
			w1 = new Circle(0,5); addChild(w1); w1.name = "w1"
			w2 = new Circle(0,5); addChild(w2); w2.name = "w2"
			collision = new Circle(0xFF0000,5); addChild(collision);
			ship = new Triangle(0x5599FF); addChild(ship);
			
			w1.x = 480; w1.y = 30;
			w2.x = 50; w2.y = 100;
			collision.x = -100; collision.y = -100;
			ship.x = stage.stageWidth >> 1; ship.y = stage.stageHeight >> 1;
			
			velo = new Vector2D(0, -3);
			
			t = new TextField(); addChild(t);
			t.x = 10; t.y = stage.stageHeight * 0.7;
			t.autoSize = TextFieldAutoSize.LEFT;
			
			stage.addEventListener(KeyboardEvent.KEY_DOWN, trigger);
			stage.addEventListener(KeyboardEvent.KEY_UP, trigger);
			stage.addEventListener(Event.ENTER_FRAME, update);
			stage.addEventListener(MouseEvent.MOUSE_DOWN, move);
			stage.addEventListener(MouseEvent.MOUSE_UP, stop);
		}
		
		private function move(e:MouseEvent):void 
		{
			if (e.target.name == "w1" || e.target.name == "w2") e.target.startDrag();
		}
		
		private function stop(e:MouseEvent):void 
		{
			if (e.target.name == "w1" || e.target.name == "w2") e.target.stopDrag();
		}
		
		private function trigger(e:KeyboardEvent):void 
		{
			if (e.type == "keyDown") {
				if (e.keyCode == Keyboard.LEFT) control = 1;
				else if (e.keyCode == Keyboard.RIGHT) control = 2;
				
				if (e.keyCode == Keyboard.UP) increase = 1;
			}
			if (e.type == "keyUp") { 
				if (e.keyCode == Keyboard.LEFT || e.keyCode == Keyboard.RIGHT) control = 0; 
				if (e.keyCode == Keyboard.UP || e.keyCode == Keyboard.DOWN)increase = 0;
			}
		}
		
		private function update(e:Event):void 
		{
			//Steering left and right
			if (control == 1)  velo = velo.rotate(Math2.radianOf(-5));
			else if (control == 2)  velo = velo.rotate(Math2.radianOf(5));
			
			//manipulating velocity
			var currVelo:Number = velo.getMagnitude();
			if (increase == 0)  {
				currVelo -= 0.5; currVelo = Math.max(currVelo, 1); //lower bound for velocity
			}
			else if (increase == 1)  {
				currVelo += 0.5; currVelo = Math.min(currVelo, 5); //upper bound for velocity
			}
			velo.setMagnitude(currVelo);
			
			//update velocity
			ship.x += velo.x; ship.y += velo.y; ship.rotation = Math2.degreeOf(velo.getAngle());
			
			//reflect when ship is out of stage
			if (ship.x <0 || ship.x > stage.stageWidth) velo.x *= -1;
			if (ship.y <0 || ship.y > stage.stageHeight) velo.y *= -1;
			
			//getting collision point
			var v1:Vector2D = new Vector2D (w1.x - w2.x, w1.y - w2.y);
			var v2:Vector2D = velo.clone();
			
			var norm_v2:Vector2D = v2.normalise();
			var norm_v1:Vector2D = v1.normalise();
			
			var a_c:Number = w2.x - ship.x;
			var b_d:Number = w2.y - ship.y;
			
			var R:Matrix = new Matrix;
			R.a = norm_v2.x; R.c = norm_v1.x;
			R.b = norm_v2.y; R.d = norm_v1.y;
			R.invert();
			
			var L:Matrix = new Matrix;
			L.a = a_c; 
			L.b = b_d;
			L.concat(R);
			
			collision.x = ship.x + L.a * norm_v2.x;
			collision.y = ship.y + L.a * norm_v2.y;
			
			//deciding if within wall segment
			var w2_collision:Vector2D = new Vector2D(collision.x - w2.x, collision.y - w2.y);
			collision.alpha = 0;
			
			//when ship is heading to left of wall
			if (w2_collision.dotProduct(v1) < 0) {
				t.text = "Ship is heading to left of wall";
			}
			else {
				//when ship is heading to right of wall
				if (w2_collision.getMagnitude() > v1.getMagnitude()) {
					t.text = "Ship is heading to right of wall"
				}
				//when ship is heading to wall segment
				else {
					var ship_collision:Vector2D = new Vector2D(collision.x - ship.x, collision.y - ship.y);
					var displacement:Number = ship_collision.getMagnitude();
					if (ship_collision.dotProduct (velo) < 0) displacement *= -1;
					
					//showing text
					var time:Number = displacement / velo.getMagnitude();
					t.text = "Frames to impact: " + time.toPrecision(3) + " frames.\n";
					time /= stage.frameRate;
					t.appendText("Time to impact: " + time.toPrecision(3) + " seconds.\n");
					
					//drop down alpha if collision had happened
					if (displacement > 0) collision.alpha = 1;
					else {
						collision.alpha = 0.5;
						t.appendText("Collision had already happened.")
					}
				}
			}
			
			//draw line of wall;
			graphics.clear();
			graphics.lineStyle(1);
			graphics.drawPath(new <int> [1, 2], new <Number>[w1.x, w1.y, w2.x, w2.y]);
		}
		
		
	}

}